function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import { NativeModules, Platform } from 'react-native';
import GeoViewport from '@mapbox/geo-viewport';
import { getMarkersCoordinates, getMarkersRegion, regionToBBox } from './utils';
const module = NativeModules.Clusterer;

if (module && typeof module.install === 'function' && !global.createSupercluster) {
  module.install();
}

const createSupercluster = global.createSupercluster;
const defaultOptions = {
  minZoom: 0,
  // min zoom to generate clusters on
  maxZoom: 16,
  // max zoom level to cluster the points on
  minPoints: 2,
  // minimum points to form a cluster
  radius: 40,
  // cluster radius in pixels
  extent: 512,
  // tile extent (radius is calculated relative to it)
  log: false,
  // whether to log timing info
  // whether to generate numeric ids for input features (in vector tiles)
  generateId: false
};
export default class SuperclusterClass {
  constructor(options) {
    _defineProperty(this, "cppInstance", undefined);

    _defineProperty(this, "options", void 0);

    _defineProperty(this, "expandCluster", clusterId => {
      if (!this.cppInstance) return {
        latitude: 0,
        longitude: 0,
        latitudeDelta: 0,
        longitudeDelta: 0
      };
      const clusterMarkersCoordinates = this.getMarkersInCluster(clusterId).map(getMarkersCoordinates);
      return getMarkersRegion(clusterMarkersCoordinates);
    });

    _defineProperty(this, "getMarkersInCluster", clusterId => {
      const clusterChildren = this.getChildren(clusterId);

      if (clusterChildren.length > 1) {
        return clusterChildren;
      }

      return this.getMarkersInCluster(clusterChildren[0].id);
    });

    if (!createSupercluster) {
      throw new Error(`The package 'react-native-clusterer' doesn't seem to be linked. Make sure: \n\n` + Platform.select({
        ios: "- You have run 'pod install'\n",
        default: ''
      }) + '- You rebuilt the app after installing the package\n' + '- You are not using Expo managed workflow\n');
    }

    this.options = { ...defaultOptions,
      ...options
    };
  }
  /**
   * Loads an array of GeoJSON Feature objects. Each feature's geometry
   * must be a GeoJSON Point. Once loaded, index is immutable.
   *
   * @param points Array of GeoJSON Features, the geometries being GeoJSON Points.
   */


  load(points) {
    this.cppInstance = createSupercluster(points, this.options);
    return this;
  }
  /**
   * Returns an array of clusters and points as `GeoJSON.Feature` objects
   * for the given bounding box (`bbox`) and zoom level (`zoom`).
   *
   * @param bbox Bounding box (`[westLng, southLat, eastLng, northLat]`).
   * @param zoom Zoom level.
   */


  getClusters(bbox, zoom) {
    if (!this.cppInstance) return [];
    return this.cppInstance.getClusters(bbox, zoom);
  }
  /**
   * Returns an array of clusters and points as `GeoJSON.Feature` objects
   * for the given bounding box (`bbox`) and zoom level (`zoom`).
   *
   * @param bbox Bounding box (`[westLng, southLat, eastLng, northLat]`).
   * @param zoom Zoom level.
   */


  getClustersFromRegion(region, mapDimensions) {
    if (!this.cppInstance) return [];
    const bbox = regionToBBox(region);
    if (region.longitudeDelta >= 40) return this.cppInstance.getClusters(bbox, this.options.minZoom);
    const viewport = GeoViewport.viewport(bbox, [mapDimensions.width, mapDimensions.height], this.options.minZoom, this.options.maxZoom, 512);
    return this.cppInstance.getClusters(bbox, viewport.zoom);
  }
  /**
   * For a given zoom and x/y coordinates, returns a
   * [geojson-vt](https://github.com/mapbox/geojson-vt)-compatible JSON
   * tile object with cluster any point features.
   */


  getTile(x, y, zoom) {
    if (!this.cppInstance) return null;
    return {
      features: this.cppInstance.getTile(x, y, zoom)
    };
  }
  /**
   * Returns the children of a cluster (on the next zoom level).
   *
   * @param clusterId Cluster ID (`cluster_id` value from feature properties).
   * @throws {Error} If `clusterId` does not exist.
   */


  getChildren(clusterId) {
    if (!this.cppInstance) return [];
    return this.cppInstance.getChildren(clusterId);
  }
  /**
   * Returns all the points of a cluster (with pagination support).
   *
   * @param clusterId Cluster ID (`cluster_id` value from feature properties).
   * @param limit The number of points to return (set to `Infinity` for all points).
   * @param offset The amount of points to skip (for pagination).
   */


  getLeaves(clusterId, limit, offset) {
    if (!this.cppInstance) return [];
    return this.cppInstance.getLeaves(clusterId, limit !== null && limit !== void 0 ? limit : 10, offset !== null && offset !== void 0 ? offset : 0);
  }
  /**
   * Returns the zoom level on which the cluster expands into several
   * children (useful for "click to zoom" feature).
   *
   * @param clusterId Cluster ID (`cluster_id` value from feature properties).
   */


  getClusterExpansionZoom(clusterId) {
    if (!this.cppInstance) return 0;
    return this.cppInstance.getClusterExpansionZoom(clusterId);
  }
  /**
   * Returns a region containing the center of all the points in a cluster
   * and the delta value by which it should be zoomed out to see all the points.
   * (usefull for animating a MapView after a cluster press).
   * @param clusterId Cluster ID (`cluster_id` value from feature properties).
   */


  /**
   * Destroy the instance.
   * @deprecated Removed in 1.2.0.
   * @returns True if cluster exists and was destroyed, else false.
   */
  destroy() {
    console.warn('React-Native-Clusterer: destroy function is no longer needed');
    return false;
  }

}
//# sourceMappingURL=Supercluster.js.map